package parser

import "strconv"
import "git.tebibyte.media/sashakoshka/fspl/lexer"
import "git.tebibyte.media/sashakoshka/fspl/entity"

var descriptionSignature = "Signature"
var startTokensSignature = []lexer.TokenKind { lexer.LBracket }

func (this *Parser) parseSignature () (*entity.Signature, error) {
	err := this.expectDesc(descriptionSignature, startTokensSignature...)
	if err != nil { return nil, err }
	err = this.expectNext(lexer.Ident)
	if err != nil { return nil, err }
	signature := &entity.Signature{
		Position: this.pos(),
		Name:     this.value(),
		
	}
	this.next()

	for {
		err := this.expectDesc (
			"Parameter or Signature end",
			appendCopy(startTokensDeclaration, lexer.RBracket)...)
		if err != nil { return nil, err }
		if this.kind() == lexer.RBracket { break }

		parameter, err := this.parseDeclaration()
		if err != nil { return nil, err }
		signature.Arguments = append(signature.Arguments, parameter)
	}
	signature.Position = signature.Position.Union(this.pos())
	this.next()

	if this.kind() == lexer.Colon {
		this.next()
		ret, err := this.parseType()
		if err != nil { return nil, err }
		signature.Return = ret
	}
	
	return signature, nil
}

func parseInteger (value string) (int64, error) {
	base := 10
	if len(value) >= 1 && value[0] == '0' {
		base = 8
		value = value[1:]
		if len(value) >= 1 {
			switch value[0] {
			case 'x':
				value = value[1:]
				base = 16
			case 'b':
				value = value[1:]
				base = 1
			}
		}
	}
	
	integer, err := strconv.ParseInt(value, base, 64)
	if err != nil { return 0, err }
	return integer, nil
}

func parseFloat (value string) (float64, error) {
	float, err := strconv.ParseFloat(value, 64)
	if err != nil { return 0, err }
	return float, nil
}
